@@ -1,6 +1,6 @@
 from django.db import models
 
-from activatable_model import BaseActivatableModel
+from activatable_model.models import BaseActivatableModel
 
 
 class ActivatableModel(BaseActivatableModel):
@@ -18,6 +18,12 @@ class ActivatableModelWRel(BaseActivatableModel):
     rel_field = models.ForeignKey(Rel, on_delete=models.PROTECT)
 
 
+class ActivatableModelWRelAndCascade(BaseActivatableModel):
+    ALLOW_CASCADE_DELETE = True
+    is_active = models.BooleanField(default=False)
+    rel_field = models.ForeignKey(Rel, on_delete=models.CASCADE)
+
+
 class ActivatableModelWNonDefaultField(BaseActivatableModel):
     ACTIVATABLE_FIELD_NAME = 'active'
     active = models.BooleanField(default=False)
