@@ -3,11 +3,18 @@ from django.core.exceptions import ValidationError
 from django.db import models
 from django.test import TestCase, TransactionTestCase
 from django_dynamic_fixture import G
-from mock import patch, MagicMock
+from mock import patch, MagicMock, call
 
-from activatable_model import BaseActivatableModel, model_activations_changed
+from activatable_model.models import BaseActivatableModel
+from activatable_model.signals import model_activations_changed, model_activations_updated
 from activatable_model.validation import get_activatable_models, validate_activatable_models
-from activatable_model.tests.models import ActivatableModel, ActivatableModelWRel, Rel, ActivatableModelWNonDefaultField
+from activatable_model.tests.models import (
+    ActivatableModel,
+    ActivatableModelWRel,
+    Rel,
+    ActivatableModelWNonDefaultField,
+    ActivatableModelWRelAndCascade,
+)
 
 
 class BaseMockActivationsSignalHanderTest(TestCase):
@@ -18,13 +25,15 @@ class BaseMockActivationsSignalHanderTest(TestCase):
         super(BaseMockActivationsSignalHanderTest, self).setUp()
         self.mock_model_activations_changed_handler = MagicMock()
         model_activations_changed.connect(self.mock_model_activations_changed_handler)
+        self.mock_model_activations_updated_handler = MagicMock()
+        model_activations_updated.connect(self.mock_model_activations_updated_handler)
 
     def tearDown(self):
         super(BaseMockActivationsSignalHanderTest, self).tearDown()
         model_activations_changed.disconnect(self.mock_model_activations_changed_handler)
 
 
-class NoCascadeTest(TransactionTestCase):
+class CascadeTest(TransactionTestCase):
     """
     Tests that cascade deletes cant happen on an activatable test model.
     """
@@ -34,6 +43,13 @@ class NoCascadeTest(TransactionTestCase):
         with self.assertRaises(models.ProtectedError):
             rel.delete()
 
+    def test_allowed_cascade(self):
+        rel = G(Rel)
+        rel_id = rel.id
+        G(ActivatableModelWRelAndCascade, rel_field=rel)
+        rel.delete()
+        self.assertEqual(ActivatableModelWRelAndCascade.objects.filter(id=rel_id).count(), 0)
+
 
 class ManagerQuerySetTest(BaseMockActivationsSignalHanderTest):
     """
@@ -80,32 +96,96 @@ class ManagerQuerySetTest(BaseMockActivationsSignalHanderTest):
         self.assertEquals(call_args[1]['sender'], ActivatableModelWNonDefaultField)
 
     def test_activate(self):
-        G(ActivatableModel, is_active=False)
-        G(ActivatableModel, is_active=True)
+        models = [
+            G(ActivatableModel, is_active=False),
+            G(ActivatableModel, is_active=True),
+        ]
         ActivatableModel.objects.activate()
         self.assertEquals(ActivatableModel.objects.filter(is_active=True).count(), 2)
-        self.assertEquals(self.mock_model_activations_changed_handler.call_count, 3)
+        static_kwargs = {
+            'sender': ActivatableModel,
+            'signal': model_activations_changed,
+        }
+        self.mock_model_activations_changed_handler.assert_has_calls([
+            call(instance_ids=[models[0].id], is_active=False, **static_kwargs),
+            call(instance_ids=[models[1].id], is_active=True, **static_kwargs),
+            call(instance_ids=[models[0].id], is_active=True, **static_kwargs),
+        ])
+        static_kwargs['signal'] = model_activations_updated
+        self.mock_model_activations_updated_handler.assert_has_calls([
+            call(instance_ids=[models[0].id], is_active=False, **static_kwargs),
+            call(instance_ids=[models[1].id], is_active=True, **static_kwargs),
+            call(instance_ids=[models[0].id, models[1].id], is_active=True, **static_kwargs),
+        ])
 
     def test_activate_custom(self):
-        G(ActivatableModelWNonDefaultField, active=False)
-        G(ActivatableModelWNonDefaultField, active=True)
+        models = [
+            G(ActivatableModelWNonDefaultField, active=False),
+            G(ActivatableModelWNonDefaultField, active=True),
+        ]
         ActivatableModelWNonDefaultField.objects.activate()
         self.assertEquals(ActivatableModelWNonDefaultField.objects.filter(active=True).count(), 2)
-        self.assertEquals(self.mock_model_activations_changed_handler.call_count, 3)
+        static_kwargs = {
+            'sender': ActivatableModelWNonDefaultField,
+            'signal': model_activations_changed,
+        }
+        self.mock_model_activations_changed_handler.assert_has_calls([
+            call(instance_ids=[models[0].id], is_active=False, **static_kwargs),
+            call(instance_ids=[models[1].id], is_active=True, **static_kwargs),
+            call(instance_ids=[models[0].id], is_active=True, **static_kwargs),
+        ])
+        static_kwargs['signal'] = model_activations_updated
+        self.mock_model_activations_updated_handler.assert_has_calls([
+            call(instance_ids=[models[0].id], is_active=False, **static_kwargs),
+            call(instance_ids=[models[1].id], is_active=True, **static_kwargs),
+            call(instance_ids=[models[0].id, models[1].id], is_active=True, **static_kwargs),
+        ])
 
     def test_deactivate(self):
-        G(ActivatableModel, is_active=False)
-        G(ActivatableModel, is_active=True)
+        models = [
+            G(ActivatableModel, is_active=False),
+            G(ActivatableModel, is_active=True),
+        ]
         ActivatableModel.objects.deactivate()
         self.assertEquals(ActivatableModel.objects.filter(is_active=False).count(), 2)
-        self.assertEquals(self.mock_model_activations_changed_handler.call_count, 3)
+        static_kwargs = {
+            'sender': ActivatableModel,
+            'signal': model_activations_changed,
+        }
+        self.mock_model_activations_changed_handler.assert_has_calls([
+            call(instance_ids=[models[0].id], is_active=False, **static_kwargs),
+            call(instance_ids=[models[1].id], is_active=True, **static_kwargs),
+            call(instance_ids=[models[1].id], is_active=False, **static_kwargs),
+        ])
+        static_kwargs['signal'] = model_activations_updated
+        self.mock_model_activations_updated_handler.assert_has_calls([
+            call(instance_ids=[models[0].id], is_active=False, **static_kwargs),
+            call(instance_ids=[models[1].id], is_active=True, **static_kwargs),
+            call(instance_ids=[models[0].id, models[1].id], is_active=False, **static_kwargs),
+        ])
 
     def test_deactivate_custom(self):
-        G(ActivatableModelWNonDefaultField, is_active=False)
-        G(ActivatableModelWNonDefaultField, is_active=True)
+        models = [
+            G(ActivatableModelWNonDefaultField, active=False),
+            G(ActivatableModelWNonDefaultField, active=True),
+        ]
         ActivatableModelWNonDefaultField.objects.deactivate()
         self.assertEquals(ActivatableModelWNonDefaultField.objects.filter(active=False).count(), 2)
-        self.assertEquals(self.mock_model_activations_changed_handler.call_count, 3)
+        static_kwargs = {
+            'sender': ActivatableModelWNonDefaultField,
+            'signal': model_activations_changed,
+        }
+        self.mock_model_activations_changed_handler.assert_has_calls([
+            call(instance_ids=[models[0].id], is_active=False, **static_kwargs),
+            call(instance_ids=[models[1].id], is_active=True, **static_kwargs),
+            call(instance_ids=[models[1].id], is_active=False, **static_kwargs),
+        ])
+        static_kwargs['signal'] = model_activations_updated
+        self.mock_model_activations_updated_handler.assert_has_calls([
+            call(instance_ids=[models[0].id], is_active=False, **static_kwargs),
+            call(instance_ids=[models[1].id], is_active=True, **static_kwargs),
+            call(instance_ids=[models[0].id, models[1].id], is_active=False, **static_kwargs),
+        ])
 
     def test_delete_no_force(self):
         G(ActivatableModel, is_active=False)
@@ -146,6 +226,10 @@ class SaveTest(BaseMockActivationsSignalHanderTest):
         self.assertEquals(call_args[1]['is_active'], False)
         self.assertEquals(call_args[1]['instance_ids'], [m.id])
         self.assertEquals(call_args[1]['sender'], ActivatableModel)
+        updated_call_args = self.mock_model_activations_updated_handler.call_args
+        self.assertEquals(updated_call_args[1]['is_active'], False)
+        self.assertEquals(updated_call_args[1]['instance_ids'], [m.id])
+        self.assertEquals(updated_call_args[1]['sender'], ActivatableModel)
 
     def test_save_not_changed(self):
         m = G(ActivatableModel, is_active=False)
@@ -153,28 +237,43 @@ class SaveTest(BaseMockActivationsSignalHanderTest):
         m.save()
 
         self.assertEquals(self.mock_model_activations_changed_handler.call_count, 1)
+        self.assertEquals(self.mock_model_activations_updated_handler.call_count, 2)
 
     def test_save_changed(self):
         m = G(ActivatableModel, is_active=False)
         m.is_active = True
         m.save()
 
+        # changed
         self.assertEquals(self.mock_model_activations_changed_handler.call_count, 2)
         call_args = self.mock_model_activations_changed_handler.call_args
         self.assertEquals(call_args[1]['is_active'], True)
         self.assertEquals(call_args[1]['instance_ids'], [m.id])
         self.assertEquals(call_args[1]['sender'], ActivatableModel)
+        # updated
+        self.assertEquals(self.mock_model_activations_updated_handler.call_count, 2)
+        updated_call_args = self.mock_model_activations_updated_handler.call_args
+        self.assertEquals(updated_call_args[1]['is_active'], True)
+        self.assertEquals(updated_call_args[1]['instance_ids'], [m.id])
+        self.assertEquals(updated_call_args[1]['sender'], ActivatableModel)
 
     def test_save_changed_custom(self):
         m = G(ActivatableModelWNonDefaultField, active=False)
         m.active = True
         m.save()
 
+        # changed
         self.assertEquals(self.mock_model_activations_changed_handler.call_count, 2)
         call_args = self.mock_model_activations_changed_handler.call_args
         self.assertEquals(call_args[1]['is_active'], True)
         self.assertEquals(call_args[1]['instance_ids'], [m.id])
         self.assertEquals(call_args[1]['sender'], ActivatableModelWNonDefaultField)
+        # updated
+        self.assertEquals(self.mock_model_activations_updated_handler.call_count, 2)
+        updated_call_args = self.mock_model_activations_updated_handler.call_args
+        self.assertEquals(updated_call_args[1]['is_active'], True)
+        self.assertEquals(updated_call_args[1]['instance_ids'], [m.id])
+        self.assertEquals(updated_call_args[1]['sender'], ActivatableModelWNonDefaultField)
 
 
 class SingleDeleteTest(BaseMockActivationsSignalHanderTest):
@@ -187,6 +286,7 @@ class SingleDeleteTest(BaseMockActivationsSignalHanderTest):
         m = ActivatableModel.objects.get(id=m.id)
         self.assertFalse(m.is_active)
         self.assertEquals(self.mock_model_activations_changed_handler.call_count, 1)
+        self.assertEquals(self.mock_model_activations_updated_handler.call_count, 2)
 
     def test_delete_no_force_active_changed(self):
         m = G(ActivatableModel, is_active=True)
@@ -194,6 +294,7 @@ class SingleDeleteTest(BaseMockActivationsSignalHanderTest):
         m = ActivatableModel.objects.get(id=m.id)
         self.assertFalse(m.is_active)
         self.assertEquals(self.mock_model_activations_changed_handler.call_count, 2)
+        self.assertEquals(self.mock_model_activations_updated_handler.call_count, 2)
 
     def test_delete_force(self):
         m = G(ActivatableModel, is_active=False)
@@ -208,7 +309,16 @@ class ValidateDbTest(TestCase):
     def test_get_activatable_models(self):
         activatable_models = get_activatable_models()
         self.assertEquals(
-            set([ActivatableModel, ActivatableModelWRel, ActivatableModelWNonDefaultField]), set(activatable_models))
+            set(
+                [
+                    ActivatableModel,
+                    ActivatableModelWRel,
+                    ActivatableModelWRelAndCascade,
+                    ActivatableModelWNonDefaultField
+                ]
+            ),
+            set(activatable_models)
+        )
 
     def test_all_valid_models(self):
         """
@@ -299,7 +409,7 @@ class ValidateDbTest(TestCase):
                 abstract = True
 
             is_active = models.BooleanField(default=False)
-            ctype = models.ForeignKey(ContentType)
+            ctype = models.ForeignKey(ContentType, on_delete=models.CASCADE)
 
         mock_get_activatable_models.return_value = [CascadableModel]
         with self.assertRaises(ValidationError):
@@ -351,8 +461,20 @@ class ValidateDbTest(TestCase):
                 abstract = True
 
             is_active = models.BooleanField(default=False)
-            ctype = models.OneToOneField(ContentType)
+            ctype = models.OneToOneField(ContentType, on_delete=models.CASCADE)
 
         mock_get_activatable_models.return_value = [CascadableModel]
         with self.assertRaises(ValidationError):
             validate_activatable_models()
+
+
+class ModelUpdatedSignalTest(BaseMockActivationsSignalHanderTest):
+    """
+    Tests the updated signal test
+    """
+    def test_no_activatable_field_updated(self):
+        m = G(ActivatableModel, is_active=False)
+        m_from_db = ActivatableModel.objects.get(id=m.id)
+        m_from_db.char_field = 'foo'
+        m_from_db.save()
+        self.assertEquals(self.mock_model_activations_updated_handler.call_count, 1)
